#include <plib.h>
#include <math.h>
#include "NU32.h"
#include <dsplib_dsp.h>
#include "StoredSignal.h"

/*************************/
/* CONSTANTS AND GLOBALS */
/*************************/
#define QFORMAT 31 // this is because we will use Q31 number format
#define SCALE_FUDGE 1000 // this is used in properly scaling the input
			 // signal.  may need to be adjusted depending
			 // on signal
#define LOG2N 10  // because our signal is 1024 long
#define LEN (1 << LOG2N) // equals 1024
#define TWIDDLE fft32c1024 // Microchip defines fft32c1024
int32c din[LEN]; // array for storing signal converted to Q31 
int32c dout[LEN]; // array for storing output of FFT
int32c twiddle[LEN / 2]; // required for FFT
int32c scratch[LEN]; // required for FFT

// We are defining the following variable as extern, because the DSP
// library actually fills in values for this variable.  Alternatively,
// we could have included "fftc.h"
extern const int32c fft32c1024[512];

/*************
 * FUNCTIONS *
 *************/
// We use the following function to convert doubles to Q31 format. The
// scale argument is used to re-scale the input signal such that it
// fits nicely in the bounds of the Q31 format.
void toQForm(double *src, int32c *dst, double scale){
  int i;
  unsigned int qform = 1 << QFORMAT;

  scale = qform / scale;
  for (i = 0; i < LEN; i++) {
    dst[i].re = (int) (src[i] * scale);
    dst[i].im = 0;
  }
}

// The following function takes the results of an FFT in Q31 format,
// and converts it into the magnitudes of the FFT as doubles.  The
// scale argument must be the same as the scale argument in the
// previous function if you want to properly extract the correct values.
void fromQForm(int32c *src, double *dst, double scale) {
  int i;
  double re, im;
  unsigned int qform = 1 << QFORMAT;
  scale = scale / qform;
  for (i = 0; i < LEN; i++) {
    re = src[i].re * scale;
    im = src[i].im * scale;
    dst[i] = sqrt(re * re + im * im);
  }
}

/********
 * MAIN *
 ********/
int main(void) {
  int i = 0;
  double scale;

  NU32_Startup();
  NU32_Initialize();

  // Find the max magnitude of the signal so that we can determine a
  // good scaling factor
  scale = 0;
  for (i = 0; i < LEN; i++)
    scale = max(scale, fabs(Signal[i]));
  scale *= SCALE_FUDGE;
  // convert signal to Q31
  toQForm(Signal, din, scale);
  // copy pre-computed twiddle factors into our twiddle array
  memcpy(twiddle, TWIDDLE, sizeof (TWIDDLE));
  // perform FFT
  mips_fft32(dout, din, twiddle, scratch, LOG2N);
  // convert Q31 results to double magnitudes
  fromQForm(dout, Signal, scale);

  // send the results to the computer
  sprintf(NU32_RS232OutBuffer, "Sending FFT:\r\n");
  NU32_WriteUART1(NU32_RS232OutBuffer);

  // send out the FFT
  for (i = 0; i < LEN / 2 + 1; i++) {
    sprintf(NU32_RS232OutBuffer, "%f\r\n", Signal[i]);
    NU32_WriteUART1(NU32_RS232OutBuffer);
  }

  // show PIC is still running
  while (1) {
    WriteCoreTimer(0);
    while (ReadCoreTimer() < 4000000);
    NU32LED1 = !NU32LED1;
    NU32LED2 = !NU32LED2;
  }

  return (0);
}
